#include "i0mapmon_topo.h"
#include "checkablecomboboxmodel.h"
#include "ui/ui_i0mapmon_topo.h"

// cumbia
#include <cumbiapool.h>
#include <cuserviceprovider.h>
#include <cumacros.h>
#include <quapps.h>
#include <cupluginloader.h>
#include <qumultireaderplugininterface.h>
// cumbia

#include <cumacros.h>
#include <QtDebug>

#ifdef QUMBIA_TANGO_CONTROLS_VERSION
#include <QRegularExpression>
#include <cutreader.h>
#include <qustring.h>
#include <quwatcher.h>
#endif

I0mapmon_topo::I0mapmon_topo(CumbiaPool *cumbia_pool, QWidget *parent) :
    QWidget(parent)
{
    // cumbia
    CuModuleLoader mloader(cumbia_pool, &m_ctrl_factory_pool, &m_log_impl);
    cu_pool = cumbia_pool;
    ui = new Ui::I0mapmon_topo;
    ui->setupUi(this, cu_pool, m_ctrl_factory_pool);

    // mloader.modules() to get the list of loaded modules
    // cumbia

    m_fel0x = 0;
    CuPluginLoader plo;
    QObject *plugin_qob;
    m_multiread_plu_i = plo.get<QuMultiReaderPluginInterface>("libcumbia-multiread-plugin.so", &plugin_qob);
    if(!m_multiread_plu_i)
        perr("I0MMonPlot: failed to load plugin \"libcumbia-multiread-plugin.so\"");
    else {
        int mode = 0;
#ifdef QUMBIA_TANGO_CONTROLS_VERSION
        mode = CuTReader::Manual;
#endif
        // concurrent reads shall be ok and avoids issues with http module
        // see plugin interface documentation notes
        m_multiread_plu_i->init(cu_pool, m_ctrl_factory_pool, QuMultiReaderPluginInterface::ConcurrentReads);

        for(int i = 1; i < qApp->arguments().size() && m_fel0x < 1; i++) {
            if(qApp->arguments()[i].contains("f01")) m_fel0x = 1;
            else if(qApp->arguments()[i].contains("f02")) m_fel0x = 2;
        }

        QString s ="Scaled";
        foreach(QString s, QStringList() << "$1/name" + s << "$1/hwPosition" + s << "$1/label" + s
                << QString("$2/FEL0%1_IOM_wavelengths").arg(m_fel0x)) {
            QuWatcher *w = new QuWatcher(this, cu_pool, m_ctrl_factory_pool);
            if(!s.endsWith("wavelengths") && !s.endsWith("hwPositionScaled")) {
                w->setSingleShot(true);
                w->setAutoDestroy(true);
            }
            if(s.endsWith("nameScaled")) w->attach(this, SLOT(m_onNamesReady(QStringList)));
            else if(s.endsWith("hwPositionScaled")) w->attach(this, SLOT(m_onHwPosReady(QStringList)));
            else if(s.endsWith("labelScaled")) w->attach(this, SLOT(m_onLabelsReady(QStringList)));
            else w->attach(this, SLOT(m_onLambdaChanged(const QVector<double>&)));
            w->setSource(s);
            if(s.endsWith("nameScaled"))
                m_dev = w->source().section('/', 0, w->source().count('/') - 1);
        }
        // scale
        connect(ui->cbLogscale, SIGNAL(toggled(bool)), ui->chart, SLOT(enableLogScale(bool)));

        connect(ui->cbBars, SIGNAL(currentIndexChanged(int)), ui->chart, SLOT(m_onBarsModeChanged(int)));
        connect(ui->rbtxf, SIGNAL(toggled(bool)), this, SLOT(m_txf(bool)));
        ui->rbIntensity->setChecked(true);
        ui->rbIntensity->setDisabled(true);
        ui->rbtxf->setDisabled(true); // enabled in m_onNamesReady

        connect(ui->lState, SIGNAL(newData(CuData)), this, SLOT(m_onStateChanged(CuData)));

        // lambda selection
        m_cb_model = nullptr;
    }
}

I0mapmon_topo::~I0mapmon_topo()
{
    m_multiread_plu_i->unsetSources();
    delete ui;
}

void I0mapmon_topo::m_onNamesReady(const QStringList &names) {
    m_names = names;
    ui->rbtxf->setEnabled(true);
    ui->rbIntensity->setEnabled(true);
    for(int i = 0 ; i < names.size(); i++) {
        printf("insertSource %s\n", QString(m_dev + "/" + names[i]).toStdString().c_str());
        m_multiread_plu_i->insertSource( m_dev + "/" + names[i], i);
    }
    connect(m_multiread_plu_i->get_qobject(), SIGNAL(onNewData(const QList<CuData> &)),
            ui->chart, SLOT(update(const QList<CuData> &)));
}

void I0mapmon_topo::m_onHwPosReady(const QStringList &names) {
    ui->chart->setLabels(names);
}

void I0mapmon_topo::m_onLabelsReady(const QStringList &names) {
    ui->chart->setNames(names);
}

void I0mapmon_topo::m_onWlenSelectionChanged(QString wlen, bool on) {
    QStringList wlens;
    QVector<int> wlens_idx;
    for(int i = 1; i < m_cb_model->rowCount(); i++) { // i = 0 is not checkable
        QStandardItem *it = m_cb_model->item(i);
        if((it->flags() & Qt::ItemIsUserCheckable) &&  it->data(Qt::CheckStateRole).toBool()) {
            wlens.append(it->text());
            wlens_idx << i - 1;
        }
    }
//    ui->leLambda->setText(wlens.join(","));
    ui->chart->setLambdas(wlens_idx);
    ui->qucbLambda->setItemText(0, wlens.join(","));
}

// update the model for the lambda selection combo box if the lambdas change
//
void I0mapmon_topo::m_onLambdaChanged(const QVector<double> &la) {
    QVector<double> current_l;
    QMap<double, bool> prev_checked;
    for(int i = 0; m_cb_model != nullptr && i < m_cb_model->rowCount(); i++) {
        printf("\e[1;35mI0mapmon_topo::m_onLambdaChanged: row count %d item %d -> %s\e[0m\n", m_cb_model->rowCount(), i, qstoc(m_cb_model->item(i)->text()));
        current_l << m_cb_model->item(i)->text().toDouble();
        prev_checked[current_l.last()] = m_cb_model->item(i)->data(Qt::CheckStateRole).toBool();
    }
    if(current_l != la) {
        QStringList wlens;
        QVector<int> wlens_idx;
        CheckableComboBoxModel *new_m = new CheckableComboBoxModel(this);
        for(int i = 0; i < la.size(); i++) {
            bool checked = m_cb_model == nullptr || m_cb_model->findItem(la.at(i)) == nullptr || m_cb_model->findItem(la.at(i))->data(Qt::CheckStateRole).toBool() || prev_checked[la[i]];
            if(checked) {
                wlens << QString::number(la.at(i));
                wlens_idx << i;
            }
            new_m->addItem(QString::number(la.at(i)), true)->setData(checked ? Qt::Checked : Qt::Unchecked, Qt::CheckStateRole);
        }
        connect(new_m, SIGNAL(itemChecked(QString,bool)), this, SLOT(m_onWlenSelectionChanged(QString, bool)));
        ui->qucbLambda->setModel(new_m);
        ui->qucbLambda->setEditable(true); // needed for older Qt
//        ui->leLambda->setText(wlens.join(","));
        m_cb_model = new_m;
        ui->chart->setLambdas(wlens_idx);
        ui->qucbLambda->setItemText(0, wlens.join(","));
    }
}

void I0mapmon_topo::m_txf(bool txf) {
    m_multiread_plu_i->unsetSources();
    int i = 0;
    foreach(QString n, m_names) {
        m_multiread_plu_i->insertSource(m_dev + "/" + n + (txf ? "_txfscaled" : ""), i++);
    }
    txf ?  ui->chart->canvas()->setPalette( QColor( "black" ))  : ui->chart->canvas()->setPalette( QColor( "white" ) );
    txf ? ui->chart->setAxisTitle( QwtPlot::xBottom, "Transmission" ) : ui->chart->setAxisTitle( QwtPlot::xBottom, "Intensity (μJ)" );
    ui->chart->setTxf(txf);
}

void I0mapmon_topo::m_onStateChanged(const CuData &da) {
    QuString s(da, "value");
    if (s.compare("ALARM", Qt::CaseInsensitive) == 0) {
        ui->lStatus->unsetSource();
        ui->lStatus->setText("Intensity of the harmonics with respect to the fundamental over threshold");
    }
    else if(ui->lStatus->source().isEmpty())
        ui->lStatus->setSource("$1->Status");
}
